-- CashStash:  Displays the amount of gold you have on all characters for a given faction / realm.  Useful to see how much gold you can access across all your alts.

-- Copyright (c) 2008 Scott Snowman (Recompense on Uldum(US)), scott.snowman@gmail.com
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
--
--    * Redistributions of source code must retain the above copyright notice, this list 
--      of conditions and the following disclaimer.
--    * Redistributions in binary form must reproduce the above copyright notice, this list
--      of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
--    * The names of its contributors may not be used to endorse 
--      or promote products derived from this software without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
-- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-- IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
-- OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
-- STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
-- EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

CashStashBank = {}
CashStash = {}

local faction, realm

local frame=CreateFrame("Button", nil, PaperDollFrame)
frame:SetFrameStrata("HIGH")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("PLAYER_MONEY")
frame:RegisterEvent("ADDON_LOADED")
frame:SetPoint("BOTTOMLEFT", 57, 40)
frame:SetHeight(58)
frame:SetWidth(70)
frame:EnableMouse(true)
frame:RegisterForClicks("LeftButtonUp")
frame:SetScript("OnClick", function() ToggleDropDownMenu(1,nil,CashStashReportMenu,PaperDollFrame,355,325) end)

local goldpicture=CreateFrame("Frame", nil, PaperDollFrame)
goldpicture:SetHeight(15)
goldpicture:SetWidth(15)
goldpicture:SetPoint("TOPLEFT", frame, "TOPLEFT", 3, -3)
local gold=frame:CreateFontString("CashStashGold","OVERLAY","NumberFontNormal")
gold:SetPoint("LEFT", goldpicture, "RIGHT", 2, 0)
gold:SetJustifyH("LEFT")
local goldtexture = goldpicture:CreateTexture(nil,"OVERLAY")
goldtexture:SetAllPoints(true)
goldtexture:SetTexture("Interface\\Icons\\INV_Misc_Coin_02")

local silverpicture=CreateFrame("Frame", nil, PaperDollFrame)
silverpicture:SetHeight(15)
silverpicture:SetWidth(15)
silverpicture:SetPoint("TOP", goldpicture, "BOTTOM", 0, -5)
local silver=frame:CreateFontString(nil,"OVERLAY","NumberFontNormal")
silver:SetPoint("LEFT",silverpicture,"RIGHT", 2, 0)
silver:SetJustifyH("LEFT")
local silvertexture = silverpicture:CreateTexture(nil,"OVERLAY")
silvertexture:SetAllPoints(true)
silvertexture:SetTexture("Interface\\Icons\\INV_Misc_Coin_04")

local copperpicture=CreateFrame("Frame", nil, PaperDollFrame)
copperpicture:SetHeight(15)
copperpicture:SetWidth(15)
copperpicture:SetPoint("TOP", silverpicture, "BOTTOM", 0, -5)
local copper=frame:CreateFontString(nil,"OVERLAY","NumberFontNormal")
copper:SetPoint("LEFT",copperpicture,"RIGHT", 2, 0)
copper:SetJustifyH("LEFT")
local coppertexture = copperpicture:CreateTexture(nil,"OVERLAY")
coppertexture:SetAllPoints(true)
coppertexture:SetTexture("Interface\\Icons\\INV_Misc_Coin_06")

function CashStash.StoreMoney()
	realm = GetRealmName()
	faction = UnitFactionGroup("player")
	local name = UnitName("player")
	local gsc = GetMoney()
	CashStashBank[name.."("..realm..")"] = { realm = realm, faction = faction, gsc = gsc }
	local sum = 0
	for _,v in pairs(CashStashBank) do
		local table = v
		if table.realm == realm and table.faction == faction then
			sum = sum + table.gsc
		end
	end
	local g, sc = math.modf(sum/10000)
	local s, c = math.modf(sc * 100)
	local c = math.floor((c * 100)+0.5)
	
	gold:SetText(g)
	silver:SetText(s)
	copper:SetText(c)
end

function CashStash.InitializeDropdown(self, level)
	if level == 1 then
		local info = {}
		info.text = "Cash Stashed: ("..realm.." - "..faction..")"
		info.isTitle = 1
		UIDropDownMenu_AddButton(info, level)
		info.isTitle = nil
		info.disabled = 0
		
		for k, v in pairs(CashStashBank) do
			local table = v
			if table.realm == realm and table.faction == faction then
				local info = {}
				local g, sc = math.modf(table.gsc/10000)
				local s, c = math.modf(sc * 100)
				local c = math.floor((c * 100)+0.5)
				info.text = tostring(k):match("[^%(]+")..": "..g.."g, "..s.."s, "..c.."c"
				UIDropDownMenu_AddButton(info, level)
			end
		end
	end
end

function CashStash.OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == "CashStash" then
		for k, v in pairs(CashStashBank) do
			if type(v) == "number" then
				CashStashBank[k] = nil
			end
		end
		
	elseif event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_MONEY" then
		CashStash.StoreMoney()
	end
end

frame:SetScript("OnEvent",CashStash.OnEvent)