-- Config start
local scale = 0.9
local anchor = "TOPLEFT"
local pos_x = 5
local pos_y = -5
local font = 'Fonts\\VisitorR.TTF'
local font_size = 14
local font_style = 'OUTLINEMONOCHROME'
-- Config end

function GetMinimapShape() return "SQUARE" end

local backdrop = {
	bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=],
	edgeFile = [=[Interface\ChatFrame\ChatFrameBackground]=], edgeSize = 1,
	insets = {top = 0, left = 0, bottom = 0, right = 0},
}

local CreateBG = function(parent)
	local bg = CreateFrame('Frame', nil, parent)
	bg:SetPoint('TOPLEFT', parent, 'TOPLEFT', -1/scale, 1/scale)
	bg:SetPoint('BOTTOMRIGHT', parent, 'BOTTOMRIGHT', 1/scale, -1/scale)
	bg:SetFrameLevel(parent:GetFrameLevel() - 1)
	bg:SetBackdrop(backdrop)
	bg:SetBackdropColor(0, 0, 0, 0.5)
	bg:SetBackdropBorderColor(0, 0, 0, 1)
	return bg
end

local frames = {
	MinimapZoomIn,
	MinimapZoomOut,
	MinimapBorder,
	MinimapBorderTop,
	MiniMapWorldMapButton,
	MinimapZoneTextButton,
	MiniMapTrackingBackground,
	MiniMapTrackingButtonBorder,
	MiniMapVoiceChatFrameBackground,
	MiniMapVoiceChatFrameBorder,
	MinimapNorthTag,
	--MiniMapInstanceDifficulty,
	--GuildInstanceDifficulty,
	MiniMapBattlefieldBorder,
	MiniMapLFGFrameBorder,
	MiniMapMailBorder,
	BattlegroundShine,
	GameTimeFrame,
}

local menuFrame = CreateFrame("Frame", "MinimapRightClickMenu", UIParent, "UIDropDownMenuTemplate")
local micromenu = {
	{text = CHARACTER_BUTTON,
	func = function() ToggleCharacter("PaperDollFrame") end},
	{text = SPELLBOOK_ABILITIES_BUTTON,
	func = function() ToggleFrame(SpellBookFrame) end},
	{text = TALENTS_BUTTON,
	func = function() if not PlayerTalentFrame then LoadAddOn("Blizzard_TalentUI") end PlayerTalentFrame_Toggle() end},
	{text = ACHIEVEMENT_BUTTON,
	func = function() ToggleAchievementFrame() end},
	{text = QUESTLOG_BUTTON,
	func = function() ToggleFrame(QuestLogFrame) end},
	{text = SOCIAL_BUTTON,
	func = function() ToggleFriendsFrame(1) end},
	{text = ACHIEVEMENTS_GUILD_TAB,
	func = function() if IsInGuild() then if not GuildFrame then LoadAddOn("Blizzard_GuildUI") end GuildFrame_Toggle() end end},
	{text = PLAYER_V_PLAYER,
	func = function() ToggleFrame(PVPFrame) end},
	{text = LFG_TITLE,
	func = function() ToggleFrame(LFDParentFrame) end},
	{text = LOOKING_FOR_RAID,
	func = function() ToggleFrame(LFRParentFrame) end},
	{text = HELP_BUTTON,
	func = function() ToggleHelpFrame() end},
}

local function OnEvent(self, event, ...)
	if event == 'PLAYER_LOGIN' then
		for _, v in pairs(frames) do
			if(v:GetObjectType() == 'Texture') then
				v:SetTexture(nil)
			else
				v:Hide()
				v.Show = function() end
			end
		end

		MiniMapInstanceDifficulty:ClearAllPoints()
		MiniMapInstanceDifficulty:SetParent(Minimap)
		MiniMapInstanceDifficulty:SetPoint("TOPRIGHT", 0, 0)
		MiniMapInstanceDifficulty:SetScale(0.70)

		LFDSearchStatus:ClearAllPoints()
		LFDSearchStatus:SetPoint("TOPLEFT", MiniMapLFGFrame, "TOPRIGHT")

		Minimap:EnableMouseWheel()
		Minimap:SetScript('OnMouseWheel', function(self, direction)
			if direction > 0 then
				MinimapZoomIn:Click()
			else
				MinimapZoomOut:Click()
			end
		end)

		MiniMapTracking:ClearAllPoints()
		MiniMapTracking:SetParent(Minimap)
		MiniMapTracking:SetPoint('TOPLEFT', -1, 1)
		MiniMapTrackingButton:SetHighlightTexture(nil)
		MiniMapTrackingIcon:SetTexCoord(0.07, 0.93, 0.07, 0.93)

		MiniMapVoiceChatFrame:ClearAllPoints()
		MiniMapVoiceChatFrame:SetParent(Minimap)
		MiniMapVoiceChatFrame:SetPoint("TOP", 0, 0)

		MiniMapLFGFrame:ClearAllPoints()
		MiniMapLFGFrame:SetParent(Minimap)
		MiniMapLFGFrame:SetPoint('BOTTOMLEFT', -2, -2)

		MiniMapBattlefieldFrame:ClearAllPoints()
		MiniMapBattlefieldFrame:SetParent(Minimap)
		MiniMapBattlefieldFrame:SetPoint('TOPRIGHT', 3, 0)

		MiniMapMailFrame:ClearAllPoints()
		MiniMapMailFrame:SetParent(Minimap)
		MiniMapMailFrame:SetPoint('BOTTOMRIGHT', 0, -4)
		MiniMapMailIcon:SetTexture("Interface\\Addons\\alMinimap\\mail")

		MinimapCluster:EnableMouse(false)

		if not IsAddOnLoaded("Blizzard_TimeManager") then
			LoadAddOn("Blizzard_TimeManager")
		end
		TimeManagerClockTicker:SetPoint('BOTTOM', Minimap, 'BOTTOM', 0, 1)
		TimeManagerClockTicker:SetFont(font, font_size/scale, font_style)
		TimeManagerClockTicker:SetShadowOffset(0, 0)
		TimeManagerClockButton:GetRegions():Hide()
		TimeManagerClockButton:SetWidth(40)
		TimeManagerClockButton:SetScript("OnClick", function(_,click)
			if click == "RightButton" then
				ToggleCalendar()
			else
				ToggleTimeManager()
			end
		end)
		self:RegisterEvent('CALENDAR_UPDATE_PENDING_INVITES')

		Minimap:ClearAllPoints()
		Minimap:SetPoint(anchor, UIParent, anchor, pos_x, pos_y)
		Minimap:SetScale(scale)
		Minimap:SetFrameStrata("LOW")
		Minimap:SetMaskTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
		Minimap.bg = CreateBG(Minimap)
		
		local zoneTextFrame = CreateFrame("Frame", nil, UIParent)
		zoneTextFrame:SetPoint("TOPLEFT", Minimap, "BOTTOMLEFT", 0, -7)
		zoneTextFrame:SetPoint("TOPRIGHT", Minimap, "BOTTOMRIGHT", 0, -7)
		zoneTextFrame:SetHeight(19)
		zoneTextFrame:SetAlpha(0)
		zoneTextFrame.bg = CreateBG(zoneTextFrame)
		
		MinimapZoneText:SetParent(zoneTextFrame)
		MinimapZoneText:ClearAllPoints()
		MinimapZoneText:SetPoint("LEFT", 2, 1)
		MinimapZoneText:SetPoint("RIGHT", -2, 1)
		MinimapZoneText:SetFont(font, font_size, font_style)
		
		Minimap:SetScript("OnEnter", function(self)
			UIFrameFadeIn(zoneTextFrame, 0.3, 0, 1)
		end)
		Minimap:SetScript("OnLeave", function(self)
			UIFrameFadeOut(zoneTextFrame, 0.3, 1, 0)
		end)

		Minimap:SetScript("OnMouseUp", function(self, button)
			if button == "MiddleButton" then
				EasyMenu(micromenu, menuFrame, "cursor", 0, 0, "MENU", 2)
			else
				Minimap_OnClick(self)
			end
		end)
	elseif event == 'CALENDAR_UPDATE_PENDING_INVITES' then
		if CalendarGetNumPendingInvites() ~= 0 then
			TimeManagerClockTicker:SetTextColor(0, 1, 0)
		else
			TimeManagerClockTicker:SetTextColor(1, 1, 1)
		end
	end
end

local addon = CreateFrame('Frame')
addon:SetScript('OnEvent', OnEvent)
addon:RegisterEvent('PLAYER_LOGIN')